from unittest.mock import patch
import pytest
from typer.testing import CliRunner
from subprocess import Popen, PIPE
from CTADIRAC.Core.Utilities.typer_callbacks import PRODUCTION_FIELDS
from CTADIRAC.ProductionSystem.scripts.cta_prod_get_all import (
    extract_user_name_from_dn,
    fill_columns,
    app,
)
from tests.unit.production import PRODUCTION_RESULTS

runner = CliRunner()


def test_extract_user_name_from_dn():
    name = "Surname Lastname"
    dn = f"/O=GRID-FR/C=FR/O=CNRS/OU=LUPM/CN={name}"
    result = extract_user_name_from_dn(dn)
    assert result == name


def test_fill_columns():
    result = fill_columns(PRODUCTION_RESULTS[0], PRODUCTION_FIELDS, True)
    assert isinstance(result, list)


@pytest.fixture
def mock_production_client():
    with patch(
        "DIRAC.ProductionSystem.Client.ProductionClient.ProductionClient"
    ) as mock_client:
        yield mock_client


def test_main(mock_production_client):
    mock_client_instance = mock_production_client.return_value
    mock_client_instance.getProductions.return_value = {
        "OK": True,
        "Value": PRODUCTION_RESULTS,
    }

    result = runner.invoke(app, ["--long", "--cond", '{"Status": "Active"}'])
    assert result.exit_code == 0
    mock_client_instance.getProductions.assert_called_once_with(
        condDict={"Status": "Active"}
    )
    assert PRODUCTION_RESULTS[0]["AuthorGroup"] in result.stdout


PRODUCTION_CONFIG = """ProdSteps:
  - ID: 1
    input_meta_query:
      parentID:
      dataset:
    job_config:
      type: MCSimulation
      version: 2020-06-29b
      array_layout: Advanced-Baseline
      site: LaPalma
      particle: electron
      pointing_dir: North
      zenith_angle: 20
      n_shower: 100
      magic:
      sct:
      moon: dark
      start_run_number: 0

Common:
  MCCampaign: Prod5bTest
  configuration_id: 15
  base_path: /vo.cta.in2p3.fr/tests/prodsys/MC"""


def run_cli(cmd):
    """Run cli command"""
    cli = Popen([cmd], stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=True)
    cli.wait()
    return cli.communicate()


def test_cta_prod_submit(tmp_path):
    production_config = tmp_path / "production_config.yml"
    with open(production_config, "w") as f:
        f.write(PRODUCTION_CONFIG)
    script_name = "cta-prod-submit"
    prod_name = "cta_prod_submit_test"
    mode = "dry-run"

    cmd = f"{script_name} {prod_name} {production_config} {mode}"
    result = run_cli(cmd)
    output, err = result
    assert err.decode("utf-8") == ""


WORKFLOW_CONFIG = """MCCampaign: PROD6
site: LaPalma
input_array_layout: Prod6-Hyperarray
processing_array_layout: Alpha
zenith: 60.0
analysis_prog_version: v0.23.0
analysis_config_version: v1
configuration_id: 16
moon: dark
pointing: North"""


def test_cta_prod_create_workflow_config(tmp_path):
    workflow_config = tmp_path / "workflow_config.yml"
    with open(workflow_config, "w") as f:
        f.write(WORKFLOW_CONFIG)

    script_name = "cta-prod-create-workflow-config"

    cmd = f"{script_name} {workflow_config}"
    result = run_cli(cmd)
    output, err = result
    assert err.decode("utf-8") == ""



#!/usr/bin/env python
"""
Wrapper of the cta-prod-split-dataset command to set metadata by chuncks of 1000
It avoids timeouts for large datasets

Usage:
   cta-prod-split-dataset-bulk <datasetName or ascii file with a list of datasets> <percentage of train_en files> <percentage of train_cl files> <percentage of test files>
   cta-prod-split-dataset-bulk <datasetName or ascii file with a list of datasets> <percentage of train_cl files> <percentage of test files>
If 3 percentages are given, it splits the dataset into 3 datasets (train_en, train_cl, test).
If 2 percentages are given, it splits the dataset into 2 datasets (train_cl, test).

Example:
  cta-prod-split-dataset-bulk Prod5b_LaPalma_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL0 0.25 0.25 0.5
Produces:
Prod5b_LaPalma_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL0_train_en
Prod5b_LaPalma_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL0_train_cl
Prod5b_LaPalma_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL0_test

  cta-prod-split-dataset-bulk Prod5b_LaPalma_AdvancedBaseline_NSB1x_proton_North_20deg_DL0 0.25 0.75
Produces:
Prod5b_LaPalma_AdvancedBaseline_NSB1x_proton_North_20deg_DL0_train_cl
Prod5b_LaPalma_AdvancedBaseline_NSB1x_proton_North_20deg_DL0_test
"""

__RCSID__ = "$Id$"


import os

import DIRAC
from DIRAC.Core.Base.Script import Script
from DIRAC import gLogger
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


@Script()
def main():
    switches, argss = Script.parseCommandLine(ignoreErrors=True)
    if len(argss) not in [3, 4]:
        Script.showHelp()
    dataset = argss[0]
    fc = FileCatalogClient()
    result = fc.getDatasetFiles(dataset)
    if not result["OK"]:
        gLogger.error("Failed to get files for dataset:", result["Message"])
        DIRAC.exit(-1)
    else:
        lfn_list = result["Value"]["Successful"][dataset]

    max_run = len(lfn_list)
    a = divmod(max_run, 1000)[0]
    if max_run > 1000:
        for i in range(0, a):
            min = i * 1000
            max = i * 1000 + 1000
            cmd = "cta-prod-split-dataset {} --min={} --max={}".format(
                " ".join(argss),
                str(min),
                str(max),
            )
            print(cmd)
            os.system(cmd)
    min = str(a * 1000)
    max = str(max_run)
    cmd = f"cta-prod-split-dataset {' '.join(argss)} --min={str(min)} --max={str(max)}"
    gLogger.notice(cmd)
    os.system(cmd)
    DIRAC.exit()


####################################################
if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
  Get the files attached to a transformation and optionally update the file status
  Extension of dirac-transformation-get-files with additional options:

  Usage:
    cta-transformation-get-files <transID or ascii file with a list of transID comma separated> <options>

  Options:
  --FileStatus : selection based on the FileStatus
  --TaskStatus : selection based on the TaskStatus
  --setFileStatus : update the FileStatus of the selected files to a new Status
  --dumpLFN : dump the list of selected LFNs in a file
  --dumpFull : dump the list of selected <TransID,LFN> in a file
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient


@Script()
def main():
    Script.registerSwitch("", "FileStatus=", "    file status")
    Script.registerSwitch("", "TaskStatus=", "    task status")
    Script.registerSwitch("", "setFileStatus=", "    new file status")
    Script.registerSwitch("", "dumpLFN", "    dump the list of selected LFNs in a file")
    Script.registerSwitch(
        "", "dumpFull", "    dump the list of selected <TransID,LFN> in a file"
    )
    switches, argss = Script.parseCommandLine(ignoreErrors=True)

    transIDList = []
    fileStatus = None
    taskStatus = None
    newFileStatus = None
    dumpLFNFlag = False
    dumpFullFlag = False

    for switch in switches:
        if switch[0] == "FileStatus":
            fileStatus = switch[1].capitalize()
        elif switch[0] == "TaskStatus":
            taskStatus = switch[1].capitalize()
        elif switch[0] == "setFileStatus":
            newFileStatus = switch[1].capitalize()
        elif switch[0] == "dumpLFN":
            dumpLFNFlag = True
        elif switch[0] == "dumpFull":
            dumpFullFlag = True

    if len(argss) != 1:
        Script.showHelp()
        DIRAC.exit(-1)

    if os.path.isfile(argss[0]):
        gLogger.notice(f"\nReading transformation IDs from input file: {argss[0]}\n")
        lines = open(argss[0]).readlines()
        transIDList += [
            transID.strip() for line in lines for transID in line.split(",")
        ]
        gLogger.notice(f"Found {len(transIDList)} transformations")
    else:
        transIDList.append(argss[0])

    if fileStatus or taskStatus:
        outputFileName = "trans{}Files_{}Tasks_{}-{}".format(
            fileStatus,
            taskStatus,
            transIDList[0],
            transIDList[-1],
        )
    else:
        outputFileName = f"transFiles_{transIDList[0]}-{transIDList[-1]}"

    if dumpLFNFlag:
        outputLFN = outputFileName + ".lfns"
        f0 = open(outputLFN, "w")

    if dumpFullFlag:
        outputFull = outputFileName + ".transID_lfns"
        f1 = open(outputFull, "w")

    tc = TransformationClient()

    for transID in transIDList:
        condDict = {"TransformationID": transID}

        if fileStatus:
            condDict.update({"Status": fileStatus})

        if taskStatus:
            res = tc.getTransformationTasks(
                {"TransformationID": transID, "ExternalStatus": taskStatus}
            )

            if not res["OK"]:
                gLogger.error(res["Message"])
                DIRAC.exit(2)

            if len(res["Value"]) == 0:
                gLogger.notice(
                    "No tasks selected for transformation %s with status %s"
                    % (transID, taskStatus)
                )

            taskIDs = []
            for task in res["Value"]:
                taskIDs.append(task["TaskID"])

            condDict.update({"TaskID": taskIDs})

        res = tc.getTransformationFiles(condDict)

        if not res["OK"]:
            gLogger.error(res["Message"])
            DIRAC.exit(2)

        if len(res["Value"]) == 0:
            gLogger.notice(
                "No files selected for transformation %s with status %s"
                % (transID, fileStatus)
            )

        transFiles = []
        for transfile in res["Value"]:
            transFiles.append(transfile["LFN"])
            if dumpLFNFlag:
                f0.write(f"{transfile['LFN']}\n")
            if dumpFullFlag:
                f1.write(f"{transID} {transfile['LFN']}\n")
            else:
                gLogger.notice(transfile["LFN"])

        if newFileStatus:
            res = tc.setFileStatusForTransformation(
                transID, newLFNsStatus=newFileStatus, lfns=transFiles
            )
            if not res["OK"]:
                gLogger.error(res["Message"])
                DIRAC.exit(2)
            gLogger.notice(f" {len(transFiles)} files set to {newFileStatus} status")

    if dumpLFNFlag:
        gLogger.notice(f"{len(transFiles)} files dumped in {outputLFN}")

    if dumpFullFlag:
        gLogger.notice(f"{len(transFiles)} files dumped in {outputFull}")


if __name__ == "__main__":
    main()



"""
    Simple and generic functions to handle inputs, files and strings for CWL interface
"""

__RCSID__ = "$Id$"

import os
import shellescape
import re


def check_if_file_input(entry):
    if isinstance(entry, dict):
        if "class" in entry:
            if entry["class"] == "File":
                return True
    else:
        return False


def add_fake_file(location):
    dir = os.path.dirname(location)
    if dir != "":
        if not os.path.exists(dir):
            os.makedirs(dir)
    open(location, "a").close()
    return


def remove_fake_file(location):
    os.remove(location)
    return


def maybe_quote(arg):
    needs_shell_quoting = re.compile(r"""(^$|[\s|&;()<>\'"$@])""").search
    return shellescape.quote(arg) if needs_shell_quoting(arg) else arg


def get_source_name(source):
    return source.rsplit("/", 1)[0]


def topological_sort(source):
    """from https://stackoverflow.com/questions/11557241/python-sorting-a-dependency-list
    perform topo sort on elements.

    :arg source: list of ``(steps, [list of dependencies])`` pairs
    :returns: list of steps, with dependencies listed first
    """
    pending = [
        (step, set(deps)) for step, deps in source
    ]  # copy deps so we can modify set in-place
    emitted = []
    while pending:
        next_pending = []
        next_emitted = []
        for entry in pending:
            step, deps = entry
            deps.difference_update(emitted)  # remove deps we emitted last pass
            if deps:  # still has deps? recheck during next pass
                next_pending.append(entry)
            else:  # no more deps? time to emit
                yield step
                emitted.append(
                    step
                )  # <-- not required, but helps preserve original ordering
                next_emitted.append(
                    step
                )  # remember what we emitted for difference_update() in next pass
        if (
            not next_emitted
        ):  # all entries have unmet deps, one of two things is wrong...
            raise ValueError(f"cyclic or missing dependency detected: {next_pending!r}")
        pending = next_pending
        emitted = next_emitted
    return emitted



from CTADIRAC.Core.Utilities.tool_box import (
    extract_run_number_from_chimp_mars,
    extract_run_number_from_corsika_sim_telarray,
    extract_run_number_from_ctapipe,
    extract_run_number_from_dl1_data_handler,
    extract_run_number_from_evndisplay,
    extract_run_number_from_image_extractor,
    extract_run_number_from_logs_tgz,
    run_number_from_filename,
)


def test_run_number_from_filename_logs_tgz():
    filename = "/path/to/file_000456.logs.tgz"
    package = "any_package"
    assert extract_run_number_from_logs_tgz(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_chimp():
    filename = "/path/to/run000456___cta.anything"
    package = "chimp"
    assert extract_run_number_from_chimp_mars(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_corsika_simhessarray():
    filename = "/path/to/run000456___cta.corsika.zst"
    package = "corsika_simhessarray"
    assert extract_run_number_from_corsika_sim_telarray(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_corsika_simtelarray():
    filename = (
        "/path/to/electron_40deg_0deg_run0000347___cta-prod6-2147m-"
        "Paranal-moon.simtel.zst"
    )
    package = "corsika_simtelarray"
    assert extract_run_number_from_corsika_sim_telarray(filename) == 347
    assert run_number_from_filename(filename, package) == 347


def test_run_number_from_filename_corsika_simtelarray_zero_padded():
    filename = (
        "/path/to/gamma_20deg_180deg_run2243___cta-prod5-paranal"
        "_desert-2147m-Paranal-dark_cone10.simtel.zst"
    )
    package = "corsika_simtelarray"
    assert extract_run_number_from_corsika_sim_telarray(filename) == 2243
    assert run_number_from_filename(filename, package) == 2243


def test_run_number_from_filename_corsika_simtelarray_tid():
    filename = "/path/to/file_tid000456.anything"
    package = "corsika_simtelarray"
    assert extract_run_number_from_corsika_sim_telarray(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_corsika_simtelarray_log():
    filename = "/path/to/run000456.log"
    package = "corsika_simtelarray"
    assert extract_run_number_from_corsika_sim_telarray(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_corsika_simtelarray_gz_log():
    filename = (
        "/path/to/proton_20deg_180deg_run17375___cta-prod5-paranal"
        "_desert-2147m-Paranal-dark.log.gz"
    )
    package = "corsika_simtelarray"
    assert extract_run_number_from_corsika_sim_telarray(filename) == 17375
    assert run_number_from_filename(filename, package) == 17375


def test_run_number_from_filename_corsika_log():
    filename = (
        "/path/to/run000900_gamma-diffuse_za20deg_azm0deg"
        "_cta-prod6-lapalma.corsika.log.gz"
    )
    package = "corsika_simtelarray"
    assert extract_run_number_from_corsika_sim_telarray(filename) == 900
    assert run_number_from_filename(filename, package) == 900


def test_run_number_from_filename_evndisplay_tid():
    filename = "/path/to/file_tid000456.anything"
    package = "evndisplay"
    assert extract_run_number_from_evndisplay(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_evndisplay_dl1_root():
    filename = "/path/to/run000456___cta.DL1.root"
    package = "evndisplay"
    assert extract_run_number_from_evndisplay(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_evndisplay_dl1_tar_gz():
    filename = "/path/to/run000456___cta.DL1.tar.gz"
    package = "evndisplay"
    assert extract_run_number_from_evndisplay(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_image_extractor():
    filename = "/path/to/srun000456-anything"
    package = "image_extractor"
    assert extract_run_number_from_image_extractor(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_dl1_data_handler():
    filename = "/path/to/runs000456-anything"
    package = "dl1_data_handler"
    assert extract_run_number_from_dl1_data_handler(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_ctapipe():
    filename = "/path/to/run000456___cta.anything"
    package = "ctapipe"
    assert extract_run_number_from_ctapipe(filename) == 456
    assert run_number_from_filename(filename, package) == 456


def test_run_number_from_filename_unknown_package():
    filename = "/path/to/000456-anything"
    package = "unknown_package"
    assert run_number_from_filename(filename, package) == -1



from unittest.mock import patch
import pytest
import typer
from CTADIRAC.Core.scripts.cta_job_get_parameter import (
    extract_parameter,
    finalize,
    get_job_list_from_job,
    get_job_list_from_transformation,
    get_jobs_parameters,
    job,
    transformation,
)


@patch("CTADIRAC.Core.scripts.cta_job_get_parameter.TransformationClient")
def test_get_job_list_from_transformation(mock_transformation_client) -> None:
    mock_transformation_client.return_value.getTransformations.return_value = {
        "OK": True,
        "Value": [{"TransformationID": 1}],
    }
    mock_transformation_client.return_value.getTransformationTasks.return_value = {
        "OK": True,
        "Value": [{"ExternalID": "1"}],
    }
    job_list = get_job_list_from_transformation(condition={"Status": ["Active"]})
    assert job_list == [1]


@patch("CTADIRAC.Core.scripts.cta_job_get_parameter.JobMonitoringClient")
def test_get_job_list_from_job(mock_job_monitoring_client) -> None:
    mock_job_monitoring_client.return_value.getJobs.return_value = {
        "OK": True,
        "Value": [1],
    }

    job_list = get_job_list_from_job(condition={"Status": ["Done"]})
    assert job_list == [1]

    mock_job_monitoring_client.return_value.getJobs.return_value = {
        "OK": False,
        "Value": [None],
    }
    job_list = get_job_list_from_job(condition={"Status": ["Done"]})
    assert job_list == []


total_cpu = "TotalCPUTime(s)"


@patch("CTADIRAC.Core.scripts.cta_job_get_parameter.JobMonitoringClient")
def test_get_jobs_parameters(mock_job_monitoring_client) -> None:
    mock_job_monitoring_client.return_value.getJobParameters.return_value = {
        "OK": True,
        "Value": {1: {total_cpu: "152"}},
    }
    jobs_parameters = get_jobs_parameters([1])
    assert jobs_parameters == {1: {total_cpu: "152"}}

    mock_job_monitoring_client.return_value.getJobParameters.return_value = {
        "OK": False,
        "Value": None,
    }
    with pytest.raises(typer.Exit) as exec_info:
        get_jobs_parameters([1])
    assert str(exec_info.value) == "{'OK': False, 'Value': None}"


@patch("CTADIRAC.Core.scripts.cta_job_get_parameter.get_jobs_parameters")
def test_extract_parameter(mock_get_job_param) -> None:
    mock_get_job_param.return_value = {1: {total_cpu: "152"}}
    param_list = extract_parameter([1])
    assert param_list == [152]

    mock_get_job_param.return_value = {
        1: {"AgentLocalSE": "CSCS-Disk", total_cpu: "152"}
    }
    param_list = extract_parameter([1], "AgentLocalSE")
    assert param_list == ["CSCS-Disk"]

    mock_get_job_param.return_value = {1: {total_cpu: "152"}}
    with pytest.raises(typer.Exit) as exec_info:
        extract_parameter([1], "AgentLocalSE")
    assert str(exec_info.value) == "Empty parameter list"


@patch("CTADIRAC.Core.scripts.cta_job_get_parameter.extract_parameter")
def test_finalize(mock_extract_param) -> None:
    mock_extract_param.return_value = []
    finalize([], "Test", "", "", False)


@patch("CTADIRAC.Core.scripts.cta_job_get_parameter.get_job_list_from_transformation")
def test_transformation_cmd(mock_get_lft) -> None:
    mock_get_lft.return_value = []
    with pytest.raises(typer.Exit) as exec_info:
        transformation(fields=["TransformationID", 152])
    assert str(exec_info.value) == "No jobs selected"


@patch("CTADIRAC.Core.scripts.cta_job_get_parameter.get_job_list_from_job")
def test_job_cmd(mock_get_lfj) -> None:
    mock_get_lfj.return_value = []
    with pytest.raises(typer.Exit) as exec_info:
        job(fields=["JobGroup", 152])
    assert str(exec_info.value) == "No jobs selected"



#!/usr/bin/env python
"""
Launch a transformation from CWL workflow descriptions

Usage:
    cta-prod-submit-from-cwl <name of the Transformation> <type of the Transformation> <path of the directory containing CWL files> <optional: dataset name> <optional: group_size>

Examples:
    cta-prod-submit-from-cwl Transformation_test MCSimulation ../CWL
    cta-prod-submit-from-cwl Transformation_test Processing ../CWL Prod5b_LaPalma_AdvancedBaseline_NSB1x_electron_North_20deg_R1 5
"""

__RCSID__ = "$Id$"

import re
import os
import DIRAC
from DIRAC.Core.Base.Script import Script
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.MCPipeJob import MCPipeJob
from CTADIRAC.Interfaces.API.CtapipeProcessJob import CtapipeProcessJob
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ
from CTADIRAC.ProductionSystem.CWL.CWLWorkflowStep import WorkflowStep


def submit_simulation_transformation(
    transfo, transfo_name, setup_cwl, setup_yml, run_cwl, run_yml, data_cwl, data_yml
):
    """Build MC Simulation Transformation"""

    # Build Transformation
    transfo.Name = "MCSimulation"
    transfo.setTransformationName(transfo_name)  # this must be unique
    transfo.setType("MCSimulation")
    transfo.setDescription("Prod6 MC Pipe TS")
    transfo.setLongDescription("Prod6 simulation pipeline")  # mandatory

    # Build WorkflowSteps
    step1 = WorkflowStep()
    step1.load(setup_cwl, setup_yml)
    step1.get_command_line()

    step2 = WorkflowStep()
    step2.load(run_cwl, run_yml)
    step2.get_command_line()
    step2.command_line = (
        "./" + step2.command_line
    )  # add ./ since dirac_prod_run is not a command

    step3 = WorkflowStep()
    step3.load(data_cwl, data_yml)
    step3.get_command_line()

    # Build Job
    MCJob = MCPipeJob()
    MCJob.setType("MCSimulation")
    MCJob.setOutputSandbox(["*Log.txt"])

    # Replace static run number with dynamic run number to run with DIRAC
    step2.command_line = re.sub(
        "--run [0-9]+", f"--run {MCJob.run_number}", step2.command_line
    )

    # Run workflow
    i_step = 1
    sw_step = MCJob.setExecutable(
        str(step1.command_line.split(" ", 1)[0]),
        arguments=str(step1.command_line.split(" ", 1)[1]),
        logFile="SetupSoftware_Log.txt",
    )
    sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
    sw_step["Value"]["descr_short"] = "Setup software"
    i_step += 1

    cs_step = MCJob.setExecutable(
        str(step2.command_line.split(" ", 1)[0]),
        arguments=str(step2.command_line.split(" ", 1)[1]),
        logFile="CorsikaSimtel_Log.txt",
    )

    cs_step["Value"]["name"] = "Step%i_CorsikaSimtel" % i_step
    cs_step["Value"]["descr_short"] = "Run Corsika piped into simtel"

    i_step += 1
    dm_step = MCJob.setExecutable(
        str(step3.command_line.split(" ", 1)[0]),
        arguments=str(step3.command_line.split(" ", 1)[1]),
        logFile="DataManagement_dark_Log.txt",
    )

    dm_step["Value"]["name"] = f"Step{i_step}_DataManagement"
    dm_step["Value"]["descr_short"] = "Save data files to SE and register them in DFC"

    MCJob.setExecutionEnv({"NSHOW": "10"})

    # Submit Transformation
    transfo.setBody(MCJob.workflow.toXML())
    result = transfo.addTransformation()  # transformation is created here
    if not result["OK"]:
        return result
    transfo.setStatus("Active")
    transfo.setAgentType("Automatic")
    return result


def submit_processing_transformation(
    transfo,
    transfo_name,
    dataset,
    group_size,
    setup_cwl,
    setup_yml,
    run_cwl,
    run_yml,
    data_cwl,
    data_yml,
):
    """Build Processing Transformation"""

    # Build Transformation
    transfo.Name = "Prod5_ctapipe_processing"
    transfo.setTransformationName(transfo_name)  # this must be unique
    transfo.setType("DataReprocessing")
    transfo.setDescription("ctapipe Modeling TS")
    transfo.setLongDescription("ctapipe Modeling processing")  # mandatory

    # Build WorkflowSteps
    step1 = WorkflowStep()
    step1.load(setup_cwl, setup_yml)
    step1.get_command_line()

    step2 = WorkflowStep()
    step2.load(run_cwl, run_yml)
    step2.get_command_line()
    step2.command_line = (
        "./" + step2.command_line
    )  # add ./ since dirac_ctapipe-process_wrapper is not a command

    step3 = WorkflowStep()
    step3.load(data_cwl, data_yml)
    step3.get_command_line()

    input_meta_query = get_dataset_MQ(dataset)

    # Build Job
    ProcessingJob = CtapipeProcessJob()
    ProcessingJob.setCPUTime(259200.0)
    ProcessingJob.setOutputSandbox(["*Log.txt"])

    # Run workflow
    i_step = 1
    sw_step = ProcessingJob.setExecutable(
        str(step1.command_line.split(" ", 1)[0]),
        arguments=str(step1.command_line.split(" ", 1)[1]),
        logFile="SetupSoftware_Log.txt",
    )
    sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
    sw_step["Value"]["descr_short"] = "Setup software"
    i_step += 1

    cs_step = ProcessingJob.setExecutable(
        str(step2.command_line.split(" ", 1)[0]),
        arguments=str(step2.command_line.split(" ", 1)[1]),
        logFile="ctapipe_stage1_Log.txt",
    )

    cs_step["Value"]["name"] = "Step%i_ctapipe_stage1" % i_step
    cs_step["Value"]["descr_short"] = "Run ctapipe stage 1"

    i_step += 1
    dm_step = ProcessingJob.setExecutable(
        str(step3.command_line.split(" ", 1)[0]),
        arguments=str(step3.command_line.split(" ", 1)[1]),
        logFile="DataManagement_Log.txt",
    )

    dm_step["Value"]["name"] = f"Step{i_step}_DataManagement"
    dm_step["Value"]["descr_short"] = "Save data files to SE and register them in DFC"

    # Submit Transformation
    transfo.setBody(ProcessingJob.workflow.toXML())
    transfo.setGroupSize(group_size)
    transfo.setInputMetaQuery(input_meta_query)
    result = transfo.addTransformation()  # transformation is created here
    if not result["OK"]:
        return result
    transfo.setStatus("Active")
    transfo.setAgentType("Automatic")
    return result


@Script()
def main():
    Script.parseCommandLine()
    argss = Script.getPositionalArgs()
    if len(argss) not in [3, 5]:
        Script.showHelp()
    transfo_name = argss[0]
    type = argss[1]
    config_dir = argss[2]
    if len(argss) >= 4:
        dataset = argss[3]
        if len(argss) == 5:
            group_size = int(argss[4])
    if type.lower() == "mcsimulation":
        transfo = Transformation()
        setup_cwl = os.path.join(config_dir, "setup-software.cwl")
        setup_yml = os.path.join(config_dir, "setup-software.yml")
        run_cwl = os.path.join(config_dir, "dirac_prod_run.cwl")
        run_yml = os.path.join(config_dir, "dirac_prod_run.yml")
        data_cwl = os.path.join(config_dir, "cta-user-managedata.cwl")
        data_yml = os.path.join(config_dir, "cta-user-managedata.yml")
        result = submit_simulation_transformation(
            transfo,
            transfo_name,
            setup_cwl,
            setup_yml,
            run_cwl,
            run_yml,
            data_cwl,
            data_yml,
        )
    elif type.lower() == "processing":
        transfo = Transformation()
        setup_cwl = os.path.join(config_dir, "setup-software.cwl")
        setup_yml = os.path.join(config_dir, "setup-software-processing.yml")
        run_cwl = os.path.join(config_dir, "dirac_ctapipe-process_wrapper.cwl")
        run_yml = os.path.join(config_dir, "dirac_ctapipe-process_wrapper.yml")
        data_cwl = os.path.join(config_dir, "cta-user-managedata.cwl")
        data_yml = os.path.join(config_dir, "cta-user-managedata-processing.yml")
        result = submit_processing_transformation(
            transfo,
            transfo_name,
            dataset,
            group_size,
            setup_cwl,
            setup_yml,
            run_cwl,
            run_yml,
            data_cwl,
            data_yml,
        )
    try:
        if not result["OK"]:
            DIRAC.gLogger.error(result["Message"])
            DIRAC.exit(-1)
        else:
            DIRAC.gLogger.notice("Done")
    except Exception:
        DIRAC.gLogger.exception()
        DIRAC.exit(-1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
from multiprocessing import Pool

# DIRAC imports
from DIRAC import gLogger
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Core.Base.Script import Script

Script.setUsageMessage(
    """
Bulk removal of a list of files
Usage:
   cta-prod-remove-file [options] <ascii file with lfn list>
"""
)

Script.parseCommandLine(ignoreErrors=True)

from DIRAC.Resources.Catalog.FileCatalog import FileCatalog

fc = FileCatalog()


def removeFile(lfn):
    res = fc.removeFile(lfn)
    if not res["OK"]:
        gLogger.error("Error removing file", lfn)
        return res["Message"]


@Script()
def main():
    args = Script.getPositionalArgs()
    if len(args) > 0:
        infile = args[0]
    else:
        Script.showHelp()

    infileList = read_inputs_from_file(infile)
    p = Pool(10)
    p.map(removeFile, infileList)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Remove a given dataset

Usage:
   cta-prod-remove-dataset <datasetName or ascii file with a list of datasets>
"""

__RCSID__ = "$Id$"

import os
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


@Script()
def main():
    Script.parseCommandLine(ignoreErrors=True)
    argss = Script.getPositionalArgs()
    fc = FileCatalogClient()
    datasetList = []
    if len(argss) == 1:
        if os.path.isfile(argss[0]):
            gLogger.notice(f"Reading datasets from input file: {argss[0]}")
            datasetList = read_inputs_from_file(argss[0])
        else:
            datasetList.append(argss[0])
    else:
        Script.showHelp()

    for dataset in datasetList:
        result = fc.removeDataset(dataset)

        if not result["OK"]:
            gLogger.error(f"Failed to remove {dataset}: {result['Message']}")
            DIRAC.exit(-1)
        else:
            gLogger.notice("Successfully removed dataset", dataset)

    DIRAC.exit()


####################################################
if __name__ == "__main__":
    main()



import datetime
from unittest.mock import patch
from rich.table import Table

from CTADIRAC.TransformationSystem.Utils.FailoverUtilities import FailoverUtilities

AGENT_NAME = "Transformation/TransformationFailoverAgent"

failover_utils = FailoverUtilities(AGENT_NAME)
# Defining TransformationClient Mock
with patch(
    "DIRAC.TransformationSystem.Client.TransformationClient.TransformationClient"
) as mock_transformation_client:
    failover_utils.trans_client = mock_transformation_client
with patch(
    "DIRAC.ProductionSystem.Client.ProductionClient.ProductionClient"
) as mock_production_client:
    failover_utils.prod_client = mock_production_client

with patch(
    "DIRAC.FrameworkSystem.Client.NotificationClient.NotificationClient"
) as mock_notification_client:
    failover_utils.notif_client = mock_notification_client

get_transformation_return_result = {
    "OK": True,
    "Value": [
        {
            "TransformationID": 448,
            "TransformationName": "00000122_Step1_MCSimulation",
            "Description": "description",
            "LongDescription": "longDescription",
            "CreationDate": datetime.datetime(2024, 1, 18, 8, 51, 41),
            "LastUpdate": datetime.datetime(2024, 1, 18, 8, 51, 43),
            "AuthorDN": "/DC=org/DC=terena/DC=tcs/C=DE/ST=Hamburg/O=Deutsches Elektronen-Synchrotron DESY/CN=majestix-vm1.zeuthen.desy.de",
            "AuthorGroup": "hosts",
            "Type": "MCSimulation",
            "Plugin": "Standard",
            "AgentType": "Automatic",
            "Status": "Active",
            "FileMask": "",
            "TransformationGroup": "General",
            "GroupSize": 1.0,
            "InheritedFrom": 0,
            "Body": "<Workflow></Workflow>\n",
            "MaxNumberOfTasks": 0,
            "EventsPerTask": 0,
            "TransformationFamily": "0",
        }
    ],
}


def test_get_eligible_transformation():
    failover_utils.trans_client.getTransformations.return_value = (
        get_transformation_return_result
    )

    result = failover_utils.get_eligible_transformation("Active", "MCSimulation")
    assert result["OK"] is True
    assert list(result["Value"].keys()) == ["448"]
    assert result["Value"]["448"] == get_transformation_return_result["Value"][0]


get_transformation_tasks_result = {
    "OK": True,
    "Value": [
        {
            "TaskID": 1,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50299",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 4, 16),
        },
        {
            "TaskID": 2,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50303",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 16, 15),
        },
        {
            "TaskID": 3,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50306",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 4, 16),
        },
        {
            "TaskID": 4,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50309",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 16, 15),
        },
        {
            "TaskID": 5,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50312",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 4, 16),
        },
        {
            "TaskID": 6,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50315",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 4, 16),
        },
        {
            "TaskID": 7,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50318",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 4, 16),
        },
        {
            "TaskID": 8,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50321",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 4, 16),
        },
        {
            "TaskID": 9,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50324",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 4, 16),
        },
        {
            "TaskID": 10,
            "TransformationID": 448,
            "ExternalStatus": "Done",
            "ExternalID": "50302",
            "TargetSE": "Unknown",
            "CreationTime": datetime.datetime(2024, 1, 18, 8, 51, 44),
            "LastUpdateTime": datetime.datetime(2024, 1, 18, 9, 4, 16),
        },
    ],
}


def test_get_transformation_tasks():
    failover_utils.trans_client.getTransformationTasks.return_value = (
        get_transformation_tasks_result
    )

    result = failover_utils.get_transformation_tasks(448)
    assert result["OK"] is True
    assert result["Value"] == get_transformation_tasks_result["Value"]

    failover_utils.trans_client.getTransformationTasks.return_value = {
        "OK": False,
        "Value": [],
        "Message": "ERROR",
    }
    result = failover_utils.get_transformation_tasks(0)
    assert result["Message"] == "ERROR"


get_transformation_files_result = {
    "OK": True,
    "Value": [
        {
            "LFN": "/vo.cta.in2p3.fr/tests/prodsys/MC/Prod5bTest/LaPalma/gamma-diffuse/sim_telarray/448/Data/000xxx/gamma_20deg_0deg_run8___cta-prod5b-lapalma_desert-2158m-LaPalma-dark_cone10.simtel.zst",
            "TransformationID": 449,
            "FileID": 2742,
            "Status": "Processed",
            "TaskID": 3,
            "TargetSE": "Unknown",
            "UsedSE": "PIC-Disk",
            "ErrorCount": 0,
            "LastUpdate": datetime.datetime(2024, 1, 18, 9, 2, 58),
            "InsertedTime": datetime.datetime(2024, 1, 18, 9, 0, 2),
        }
    ],
}


def test_get_transformation_files():
    failover_utils.trans_client.getTransformationFiles.return_value = (
        get_transformation_files_result
    )

    result = failover_utils.get_transformation_files({"TransformationID": 449})
    assert result == get_transformation_files_result


get_transformation_stats_result = {
    "OK": True,
    "Value": {"Processed": 10, "Total": 10},
    "rpcStub": (
        (
            "Transformation/TransformationManager",
            {"timeout": 600, "useAccessToken": False},
        ),
        "getTransformationStats",
        [449],
    ),
}


def test_is_transformation_processed():
    failover_utils.trans_client.getTransformationStats.return_value = (
        get_transformation_stats_result
    )

    result = failover_utils.is_transformation_processed(449)
    assert result is True

    failover_utils.trans_client.getTransformationStats.return_value = {
        "OK": True,
        "Value": {"Processed": 6, "Total": 10},
    }

    result = failover_utils.is_transformation_processed(449)
    assert result is False

    failover_utils.trans_client.getTransformationStats.return_value = {
        "OK": False,
        "Value": [],
    }
    result = failover_utils.is_transformation_processed(449)
    assert result is False


complete_transformation_result = {
    "OK": True,
    "Value": 0,
    "rpcStub": (
        (
            "Transformation/TransformationManager",
            {"timeout": 600, "useAccessToken": False},
        ),
        "setTransformationParameter",
        [466, "Status", "Completed"],
    ),
}


def test_complete_transformation():
    failover_utils.trans_client.completeTransformation.return_value = (
        complete_transformation_result
    )

    result = failover_utils.complete_transformation(466)
    assert result["Value"]["Message"] == "[466] Transformation Completed."

    failover_utils.trans_client.completeTransformation.return_value = {
        "OK": False,
        "Value": None,
    }
    result = failover_utils.complete_transformation(466)
    assert "[466]: Can't complete transformation" in result["Message"]


def test_get_parent_transformation_id():
    failover_utils.prod_client.get_parent_transformation.return_value = {
        "OK": True,
        "Value": [[-1]],
    }
    result = failover_utils.get_parent_transformation_id(466)
    assert result == -1

    failover_utils.prod_client.get_parent_transformation.return_value = {
        "OK": True,
        "Value": [],
    }
    result = failover_utils.get_parent_transformation_id(466)
    assert result is None

    failover_utils.prod_client.get_parent_transformation.return_value = {
        "OK": False,
        "Value": [],
    }
    result = failover_utils.get_parent_transformation_id(466)
    assert result is None


def test_get_parent_type():
    parent_id = 123
    mock_response = {"OK": True, "Value": [{"Type": "MockParentType"}]}

    failover_utils.trans_client.getTransformations.return_value = mock_response
    parent_type = failover_utils.get_parent_type(parent_id)

    assert parent_type == "MockParentType"


@patch("CTADIRAC.TransformationSystem.Utils.FailoverUtilities.Transformation")
def test_set_task_status(mock_transformation_class):
    mock_transformation_instance = mock_transformation_class.return_value

    # Define test data
    trans_id = 123
    task_id = 456
    status = "Completed"

    # Call the method under test
    failover_utils.set_task_status(trans_id, task_id, status)

    # Assertions to check if the right methods were called with the correct arguments
    mock_transformation_class.assert_called_once_with(
        transID=trans_id, transClient=failover_utils.trans_client
    )
    mock_transformation_instance.setTaskStatus.assert_called_once_with(
        taskID=task_id, status=status
    )


def test_sorting_key():
    sample_data = [("b", 2, "foo"), ("a", 3, "bar"), ("c", 1, "baz")]

    expected_sorted_result = [("a", 3, "bar"), ("b", 2, "foo"), ("c", 1, "baz")]

    sorted_data = sorted(sample_data, key=failover_utils.sorting_key(0, 1))

    assert sorted_data == expected_sorted_result


def test_create_mail_message():
    trans_type = "Processing"
    trans_id = 466
    trans_report: dict[str, dict[str, str]] = {trans_type: {f"{trans_id}": "Message"}}
    # Test html
    result: str = failover_utils.create_mail_message(trans_report, html=True)
    assert isinstance(result, str)
    assert '<span class="r1">' in result
    for line in result.split("\n"):
        if '<span class="r1">' in line:
            report_header: str = line.split('<span class="r1">')[1].split("<")[0]
            assert report_header == "Report on Processing transformations:"

    # Test text
    result: str = failover_utils.create_mail_message(trans_report)
    assert isinstance(result, str)
    assert "Report on Processing transformations:Message\n" == result


def test_init_rich_table():
    result: Table = failover_utils.init_rich_table("Test", ["f1", "f2"])
    assert isinstance(result, Table)


def test_generate_table():
    table: Table = failover_utils.init_rich_table("Test", ["f1", "f2"])
    # Test html
    result: str = failover_utils.generate_table(table, html=True)
    header: str = result.split("\n")[0]
    foot: str = result.split("\n")[-2]
    assert header == "<!DOCTYPE html>"
    assert foot == "</html>"

    # Test text
    result: str = failover_utils.generate_table(table)
    header: str = result.split("\n")[0]
    foot: str = result.split("\n")[-2]
    assert header == "   Test    "
    assert foot == "└────┴────┘"


def test_send_mail():
    failover_utils.notif_client.sendMail.return_value = {"OK": True, "Value": "Message"}
    failover_utils.send_mail("Subject", "Message")



# Python code to demonstrate working of unittest
import unittest


class TestStringMethods(unittest.TestCase):
    def setUp(self):
        pass

    # Returns True if the string contains 4 a.
    def test_strings_a(self):
        self.assertEqual("a" * 4, "aaaa")

    # Returns True if the string is in upper case.
    def test_upper(self):
        self.assertEqual("foo".upper(), "FOO")

    # Returns TRUE if the string is in uppercase
    # else returns False.
    def test_isupper(self):
        self.assertTrue("FOO".isupper())
        self.assertFalse("Foo".isupper())

    # Returns true if the string is stripped and
    # matches the given output.
    def test_strip(self):
        s = "geeksforgeeks"
        self.assertEqual(s.strip("geek"), "sforgeeks")

    # Returns true if the string splits and matches
    # the given output.
    def test_split(self):
        s = "hello world"
        self.assertEqual(s.split(), ["hello", "world"])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
Split a dataset into train/test datasets.
Usage:
   cta-prod-split-dataset <datasetName or ascii file with a list of datasets> <percentage of train_en files> <percentage of train_cl files> <percentage of test files>
   cta-prod-split-dataset <datasetName or ascii file with a list of datasets> <percentage of train_cl files> <percentage of test files>
If 3 percentages are given, it splits the dataset into 3 datasets (train_en, train_cl, test).
If 2 percentages are given, it splits the dataset into 2 datasets (train_cl, test).

Example:
  cta-prod-split-dataset Prod5b_LaPalma_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL0 0.25 0.25 0.5
Produces:
Prod5b_LaPalma_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL0_train_en
Prod5b_LaPalma_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL0_train_cl
Prod5b_LaPalma_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL0_test

  cta-prod-split-dataset Prod5b_LaPalma_AdvancedBaseline_NSB1x_proton_North_20deg_DL0 0.25 0.75
Produces:
Prod5b_LaPalma_AdvancedBaseline_NSB1x_proton_North_20deg_DL0_train_cl
Prod5b_LaPalma_AdvancedBaseline_NSB1x_proton_North_20deg_DL0_test
"""

__RCSID__ = "$Id$"


import os

import DIRAC
from DIRAC.Core.Base.Script import Script
from DIRAC import gLogger
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


def set_split_meta_data(lfn_list, meta_value, timeout):
    pathMetaDict = {}
    metadata = {"split": meta_value}
    for lfn in lfn_list:
        pathMetaDict[lfn] = metadata
    fc = FileCatalogClient()
    result = fc.setMetadataBulk(pathMetaDict, timeout=timeout)
    if not result["OK"]:
        gLogger.error(result["Message"])
        DIRAC.exit(-1)
    return


def get_dataset_meta_query(dataset_name):
    fc = FileCatalogClient()
    res = fc.getDatasets(dataset_name)
    if not res["OK"]:
        gLogger.error("Failed to get datasets")
        DIRAC.exit(-1)
    dataset_dict = res["Value"]
    res = dataset_dict["Successful"][dataset_name][dataset_name]
    meta_query = res["MetaQuery"]
    return meta_query


def add_split_dataset(dataset_name, dataset_meta_query, split_value):
    fc = FileCatalogClient()
    dataset_meta_query["split"] = split_value
    dataset_split = f"{dataset_name}_{split_value}"
    datasetPath = os.path.join("/vo.cta.in2p3.fr/datasets/", dataset_split)
    res = fc.addDataset({datasetPath: dataset_meta_query})
    if not res["OK"]:
        gLogger.error(f"Failed to add dataset {dataset_split}: {res['Message']}")
        DIRAC.exit(-1)

    if dataset_split in res["Value"]["Failed"]:
        gLogger.error(
            "Failed to add dataset %s: %s"
            % (dataset_split, res["Value"]["Failed"][dataset_split])
        )
        DIRAC.exit(-1)
    return res


@Script()
def main():
    Script.registerSwitch("", "timeout=", "    timeout")
    Script.registerSwitch("", "min=", "    min of the full list")
    Script.registerSwitch("", "max=", "    max of the full list")
    switches, argss = Script.parseCommandLine(ignoreErrors=True)

    timeout = 600
    min = None
    max = None
    for switch in switches:
        if switch[0] == "timeout":
            timeout = int(switch[1])
        if switch[0] == "min":
            min = int(switch[1])
        if switch[0] == "max":
            max = int(switch[1])

    if len(argss) not in [3, 4]:
        Script.showHelp()

    dataset = argss[0]
    if len(argss) == 4:
        split_en = float(argss[1])
        split_cl = float(argss[2])
        split_test = float(argss[3])
        if split_en + split_cl + split_test != 1:
            gLogger.error("The sum of percentages must be equal to 1")
            DIRAC.exit(-1)
    elif len(argss) == 3:
        split_cl = float(argss[1])
        split_test = float(argss[2])
        if split_cl + split_test != 1:
            gLogger.error("The sum of percentages must be equal to 1")
            DIRAC.exit(-1)

    fc = FileCatalogClient()
    result = fc.getDatasetFiles(dataset)
    if not result["OK"]:
        gLogger.error("Failed to get files for dataset:", result["Message"])
        DIRAC.exit(-1)
    else:
        lfn_list = result["Value"]["Successful"][dataset]

    max_run = len(lfn_list)
    dataset_mq = get_dataset_meta_query(dataset)

    if len(argss) == 4:
        train_en_max_run = int(max_run * split_en)
        train_cl_max_run = int(max_run * split_cl)
        train_en_list = lfn_list[0:train_en_max_run]
        train_cl_list = lfn_list[
            train_en_max_run : (train_en_max_run + train_cl_max_run)
        ]
        test_list = lfn_list[(train_en_max_run + train_cl_max_run) :]

        if min is not None and max is not None:
            if min < train_en_max_run:
                set_split_meta_data(train_en_list[min:max], "train_en", timeout)
            if min < len(train_cl_list):
                set_split_meta_data(train_cl_list[min:max], "train_cl", timeout)
            if min < len(test_list):
                set_split_meta_data(test_list[min:max], "test", timeout)
        else:
            set_split_meta_data(train_en_list, "train_en", timeout)
            set_split_meta_data(train_cl_list, "train_cl", timeout)
            set_split_meta_data(test_list, "test", timeout)
        add_split_dataset(dataset, dataset_mq, "train_en")
        add_split_dataset(dataset, dataset_mq, "train_cl")
        add_split_dataset(dataset, dataset_mq, "test")
        gLogger.notice(
            f"Successfully split dataset {dataset} into 3 datasets : \n"
            f"{dataset}_train_en \n{dataset}_train_cl \n{dataset}_test"
        )
        DIRAC.exit()

    elif len(argss) == 3:
        train_cl_max_run = int(max_run * split_cl)
        train_cl_list = lfn_list[0:train_cl_max_run]
        test_list = lfn_list[train_cl_max_run:]
        if min is not None and max is not None:
            if min < train_cl_max_run:
                set_split_meta_data(train_cl_list[min:max], "train_cl", timeout)
            if min < len(test_list):
                set_split_meta_data(test_list[min:max], "test", timeout)
        else:
            set_split_meta_data(train_cl_list, "train_cl", timeout)
            set_split_meta_data(test_list, "test", timeout)
        add_split_dataset(dataset, dataset_mq, "train_cl")
        add_split_dataset(dataset, dataset_mq, "test")
        gLogger.notice(
            f"Successfully split dataset {dataset} into 2 datasets : \n{dataset}_train_cl \n{dataset}_test"
        )
        DIRAC.exit()


####################################################
if __name__ == "__main__":
    main()
